﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2015 Joseph Helm and Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Problems.BinPacking.Interfaces;

namespace HeuristicLab.Encodings.PackingEncoding.GroupingVector {
  [Item("Grouping Vector Crossover", "An operator which crosses two GroupingVector representations.")]
  [StorableClass]
  public abstract class GroupingVectorCrossover : PackingSolutionCrossover, IGroupingVectorOperator {

    [StorableConstructor]
    protected GroupingVectorCrossover(bool deserializing) : base(deserializing) { }
    protected GroupingVectorCrossover(GroupingVectorCrossover original, Cloner cloner) : base(original, cloner) { }
    public GroupingVectorCrossover()
      : base() {
        ParentsParameter.ActualName = "GroupingVector";
        ChildParameter.ActualName = "GroupingVector";
    }

    public abstract GroupingVectorEncoding Cross(IRandom random, GroupingVectorEncoding parent1, GroupingVectorEncoding parent2);

    public override IOperation Apply() {
      var parents = ParentsParameter.ActualValue;

      ChildParameter.ActualValue =
        Cross(RandomParameter.ActualValue, parents[0] as GroupingVectorEncoding, parents[1] as GroupingVectorEncoding);

      return base.Apply();
    }
  }
}
